##  tar.R
##  
##  written by:
##
##  Bruce E. Hansen
##  Department of Economics
##  Social Science Building
##  University of Wisconsin
##  Madison, WI 53706-1393
##  behansen@wisc.edu
##  http://www.ssc.wisc.edu/~bhansen/
##   
##  This is a R program file. It estimates a TAR model for
##  U.S. umemployment. It calls the R procedure thr_est.R.
##  See this file for further documentation.

source("Z:\\thr_est.R")

graph <- 1        # Set graph=1 to view the graph else set set graph=0
omit <- 0         # lags omitted from autoregression, if all included set omit=0       
h <- 1            # set h=1 for heteroskedasticity-correction else set h=0                     
name <- "UR"      # name of series                   
trim_per <- .15   # percentage of sample to trim from ends for test, e.g., trim_per=.15     
p <- 12           # autoregressive order             

# Load Data into vector y #
u <- read.table("z:\\lhmu.dat")
c <- read.table("z:\\lhmc.dat")
u <- as.matrix(u)
c <- as.matrix(c)
t1 <- floor(u[1])
ts <- 1959
s1 <- (ts-t1)*12+3
n <- nrow(u)
y <- as.matrix((u[s1:n]/c[s1:n])*100)
n <- nrow(y)


# Graph #
x11()
xt <- as.matrix(seq(ts+1/12,ts+n/12,by=1/12))
plot(xt,y,type="l",ann=0)
title(xlab="Unemployment Rate, Men 20+ Years")


# Taking First Differences #
dy <- as.matrix(y[2:n]-y[1:(n-1)])

# Create data matrix (lags, etc) #

rd <- 1000
n <- nrow(dy)
dat <- dy[(1+p):n]
names <- name
for (j in 1:p){
    if (sum(omit==j)==0){
        dat <- cbind(dat,dy[(1+p-j):(n-j)])
        if (j<10){ pn <- paste(c("0"),j,sep="")
        }else{ pn <- as.character(j)}
        namej <- paste(c("DY(t-"),pn,c(")"),sep="")
        names <- rbind(names,namej)    
    }  
}
xi <- seq(2,ncol(dat),by=1)
d <- 12
q <- y[(p+1):n] - y[(p+1-d):(n-d)]
dat = cbind(dat,q)
if (d<10) { pn <- paste(c("0"),d,sep="")
}else{ pn <- as.character(d)}
namej <- paste(c("Y*(t-"),pn,c(")"),sep="")
names <- rbind(names,namej)    


##############################################################

# Spectral Density Analysis  #

ar1 <- matrix(c( -0.18598887,
                  0.084300567,
                  0.13224909,
                  0.16545326,
                  0.069956299,
                  0.026875568,
                  0.061981325,
                  0.044455321,
                 -0.031329244,
                 -0.057287164,
                  0.090689689,
                 -0.13584895),12,1)
sig1 <- 0.024656225 

ar2 <- matrix(c( 0.24120656,
                 0.24083988,
                 0.12358546,
                -0.026373618,
                -0.019790247,
                -0.084133292,
                -0.15050268,
                -0.034745770,
                 0.092079241,
                 0.10290617,
                -0.11438944,
                -0.41237342),12,1)
sig2 <- 0.038820986
c1 <- rbind(as.matrix(-rev(ar1)),1)
c2 <- rbind(as.matrix(-rev(ar2)),1)

n <- 500
omegas <- as.matrix(seq(0,(n-1)*pi/n,by=pi/n))
sp1 <- matrix(c(0),n,1) 
sp2 <- matrix(c(0),n,1) 

omegai <- 1 
while (omegai<=n){

    omega <- omegas[omegai]
    eo1 <- exp(omega*1i);
    eo2 <- exp(-omega*1i);

    eecc11 <- 0
    eecc21 <- 0
    eecc12 <- 0
    eecc22 <- 0
    ind <- 0
    for (i in 1:length(c1)){
        eecc11 <- eecc11+c1[i]*(eo1^(nrow(c1)-i))
        eecc21 <- eecc21+c1[i]*(eo2^(nrow(c1)-i))
        eecc12 <- eecc12+c2[i]*(eo1^(nrow(c1)-i))
        eecc22 <- eecc22+c2[i]*(eo2^(nrow(c1)-i))
    }
    
    sp1[omegai] <- (sig1/(2*pi))/(eecc11*eecc21)
    sp2[omegai] <- (sig2/(2*pi))/(eecc12*eecc22)

    omegai <- omegai+1
}
sp1 <- Re(sp1)
sp2 <- Re(sp2)

x11()
xxlim <- range(omegas)
yylim <- c(0,0.04)
plot(omegas,sp1,lty=1,col=1,xlim=xxlim,ylim=yylim,type="l",ann=0)
lines(omegas,sp2,lty=2,col=2)     
title(xlab="Spectral Density by Regime") 
legend("topright",c("Regime 1","Regime 2"),lty=c(1,2),col=c(1,2))


##############################################################

# Classification of Regimes  #

q1 <- .212918
q2 <- .339885
i1 <- (q<q1)
i2 <- (q<=q2)-i1
i3 <- 1-i1-i2
yp <- y[(p+2):nrow(y)]

d1 <- yp*i1-(1-i1)
d2 <- yp*i2-(1-i2)
d3 <- yp*i3-(1-i3)
d <- cbind(d1,d2,d3)

n <- nrow(d)

x11()
st <- 12*10
xt <- as.matrix(seq(ts+1+2/12,ts+1+1/12+n/12,by=1/12))
xxlim <- c(1968,1998)
yylim <- c(2,10)
plot(xt[st:n],d[st:n,1],pch=2,col=3,xlim=xxlim,ylim=yylim,type="p",ann=0)
points(xt[st:n],d[st:n,2],pch=1,col=4)
points(xt[st:n],d[st:n,3],pch=3,col=2) 
title(xlab="Classification by Regime",ylab="Unemployment Rate") 
legend("bottomright",c("Regime 1","Uncertain","Regime 2"),pch=c(2,1,3),col=c(3,4,2))

##############################################################

qi = ncol(dat)
cat("Sample Period ", ts,"~",floor(xt[n]),sep="","\n", "\n")
out1 <- thr_est(dat,names,1,xi,qi,1)
save.image(file = "tarout.RData")


